@@ -1,7 +1,7 @@
 import pytz
 
 from datetime import datetime, timedelta
-from typing import Dict, Union, List
+from typing import Dict, Union, List, Optional, Iterable
 from functools import partial
 from beartype import beartype
 
@@ -21,13 +21,16 @@ __all__ = (
     "TimingsDateArg",
     "CalendarDateArg",
     "DefaultArgs",
+    "Qibla",
+    "Ism",
 )
 
 
 class Tune:
     """
     Represents a Tune obj that is returned from API.
-    Can be used to make an obj that will be used as a tune param in :class:`DefaultArgs`
+    Can be used to make an obj that will be used as a tune param in \
+    :class:`DefaultArgs`
 
     Attributes
     ----------
@@ -73,7 +76,9 @@ class Tune:
 
     @property
     def value(self):
-        """:class:`str`: The string value that will be used to get response."""
+        """:class:`str`: The string value that will be used to get response.
+
+        Format: imsak,fajr,sunrise,dhuhr,asr,maghrib,sunset,isha,midnight"""
         return (
             "{0.imsak},{0.fajr},{0.sunrise},{0.dhuhr},{0.asr},"
             "{0.maghrib},{0.sunset},{0.isha},{0.midnight}".format(self)
@@ -99,17 +104,86 @@ class Tune:
         return hash(self.value)
 
 
+class Qibla:
+    """Represents a Qibla obj.
+
+    Do not create this class yourself. Only get it through a getter.
+
+    Attributes
+    ----------
+        longitude: :class:`float`
+            Longitude coordinate.
+        latitude: :class:`float`
+            Latitude coordinate.
+        direction: :class:`float`
+            Qibla direction.
+
+    *New in v0.1.3*
+    """
+
+    def __init__(self, longitude: float, latitude: float, direction: float):
+        self.longitude = longitude
+        self.latitude = latitude
+        self.direction = direction
+
+    def __repr__(self):  # pragma: no cover
+        return "<Qibla longitude={0.longitude} latitude={0.latitude}>".format(
+            self
+        )
+
+    def __hash__(self):  # pragma: no cover
+        return hash((self.longitude, self.latitude, self.direction))
+
+
+class Ism:
+    """Represents an Ism obj.
+
+    Do not create this class yourself. Only get it through a getter.
+
+    Attributes
+    ----------
+        name: :class:`str`
+            The name in arabic.
+        transliteration: :class:`str`
+            The transliteration of the name.
+        number: :class:`int`
+            Ism's number/id.
+        en: :class:`str`
+            The name in english.
+
+    *New in v0.1.3*
+    """
+
+    def __init__(self, name: str, transliteration: str, number: int, en: dict):
+        self.name = name
+        self.transliteration = transliteration
+        self.number = number
+        self.en = en["meaning"]
+
+    def __repr__(self):  # pragma: no cover
+        return "<Ism name={0.name} en={0.en}>".format(self)
+
+    def __hash__(self):  # pragma: no cover
+        return hash(self.name)
+
+
 class Schools:
+    """Available schools"""
+
     STANDARD = SHAFI = 0
     HANAFI = 1
 
 
 class MidnightModes:
+    """Available midnight modes"""
+
     STANDARD = 0
     JAFARI = 1
 
 
 class LatitudeAdjustmentMethods:
+    """Available latitude adjustment methods"""
+
     MIDDLE_OF_THE_NIGHT = 1
     ONE_SEVENTH = 2
     ANGLE_BASED = 3
@@ -123,19 +197,23 @@ class Prayer:
     Attributes
     ----------
         timings: :class:`Timings`
-            Original Timings obj. New in v0.1.2
+            Original Timings obj.
         name: :class:`str`
             Prayer name.
-        time: :class:`datetime`
+        time: :class:`datetime.datetime`
             Prayer's time.
-        time_utc: Optional[:class:`datetime`]
+        time_utc: Optional[:class:`datetime.datetime`]
             Prayer's time in utc, might be None when time doesn't exist
-            because of a daylight savings switch. New in v0.1.2
+            because of a daylight savings switch.
         str_time: :class:`str`
             Better looking string format for prayer's time.
+
+    *New in v0.1.2: timings, time_utc*
     """
 
-    def __init__(self, name: str, time: str, timings: "Timings", date: str = None):
+    def __init__(
+        self, name: str, time: str, timings: "Timings", date: str = None
+    ):
         self.timings = timings
         self.name = name
         if date is None:
@@ -144,20 +222,27 @@ class Prayer:
         time = time.split()[0]
         self.time = datetime.strptime(time, "%H:%M").replace(year, month, day)
         try:
-            self.time_utc = self.time + timings.data.meta.timezone.utcoffset(self.time)
+            self.time_utc = self.time + timings.data.meta.timezone.utcoffset(
+                self.time
+            )
         except pytz.exceptions.NonExistentTimeError:  # pragma: no cover
             self.time_utc = None
         self.str_time = self.time.strftime("%H:%M %d-%m-%Y")
 
     @property
     def remaining(self):
-        """:class:`timedelta`: remaining time for prayer."""
+        """:class:`datetime.timedelta`: remaining time for prayer.
+
+        *New in v0.1.2*
+        """
         return self.time - datetime.utcnow()
 
     @property
     def remaining_utc(self):
-        """Optional[:class:`timedelta`]: remaining time for prayer for utc.
-        New in v0.1.2"""
+        """Optional[:class:`datetime.timedelta`]: remaining time for prayer for utc.
+
+        *New in v0.1.2*
+        """
         return self.time_utc and self.time_utc - datetime.utcnow()
 
     def __repr__(self):  # pragma: no cover
@@ -197,14 +282,14 @@ class CalendarDateArg:
             "true" if month was not given otherwise "false".
 
         hijri: :class:`bool`
-            Whether `year` is a hijri year or not.
+            Whether `year` given is a hijri year or not.
     """
 
     @beartype
     def __init__(
         self,
         year: int,
-        month: int = None,
+        month: Optional[int] = None,
         hijri: bool = False,
     ):
         # TODO: check for year limits
@@ -227,6 +312,9 @@ class CalendarDateArg:
     def as_dict(self):
         return {"year": self.year, "annual": self.annual, "month": self.month}
 
+    def __hash__(self):  # pragma: no cover
+        return hash((self.year, self.annual, self.month))
+
 
 class TimingsDateArg:
     """
@@ -234,7 +322,7 @@ class TimingsDateArg:
 
     Parameters
     ----------
-        date: Optional[:class:`int` or :class:`str` or :class:`datetime`]
+        date: Optional[:class:`int` or :class:`str` or :class:`datetime.datetime`]
             Can be either int representing the UNIX format or a str in
             DD-MM-YYYY format or a datetime obj.
             Default: current date.
@@ -247,7 +335,7 @@ class TimingsDateArg:
     """
 
     @beartype
-    def __init__(self, date: Union[str, int, datetime] = None):
+    def __init__(self, date: Optional[Union[str, int, datetime]] = None):
         if date is None:
             date = datetime.utcnow()
         elif isinstance(date, int):
@@ -266,6 +354,9 @@ class TimingsDateArg:
 
         self.date = date  # noqa
 
+    def __hash__(self):  # pragma: no cover
+        return hash(self.date)
+
 
 class DefaultArgs:
     """
@@ -273,11 +364,12 @@ class DefaultArgs:
 
     Parameters
     ----------
-        method: :class:`Method` or :class:`int`
-            A prayer time calculation method, you can look into all methods from :meth:`AsyncClient.get_all_methods()`.
+        method: :class:`methods.Method` or :class:`int`
+            A prayer time calculation method, you can look into all methods \
+            from :meth:`AsyncClient.get_all_methods()`.
             Default: ISNA (Islamic Society of North America).
 
-        tune: :class:`Tune`
+        tune: Optional[:class:`Tune`]
             To offset returned timings.
             Default: Tune()
 
@@ -286,7 +378,8 @@ class DefaultArgs:
             Default: Shafi
 
         midnightMode: :class:`int`
-            0 for Standard (Mid Sunset to Sunrise), 1 for Jafari (Mid Sunset to Fajr).
+            0 for Standard (Mid Sunset to Sunrise), 1 for Jafari (Mid Sunset \
+            to Fajr).
             Default: Standard
 
         latitudeAdjustmentMethod: :class:`int`
@@ -318,11 +411,13 @@ class DefaultArgs:
         adjustment: :class:`int`
     """
 
+    # TODO: add timezone param
+
     @beartype
     def __init__(
         self,
         method: Union[Method, int] = ISNA,
-        tune: Tune = None,
+        tune: Optional[Tune] = None,
         school: int = Schools.SHAFI,
         midnightMode: int = MidnightModes.STANDARD,  # noqa
         latitudeAdjustmentMethod: int = LatitudeAdjustmentMethods.ANGLE_BASED,  # noqa
@@ -333,7 +428,9 @@ class DefaultArgs:
             method = method.id
 
         if method not in range(16):  # pragma: no cover
-            raise ValueError("Expected method in 0-15 range" " got {!r}".format(method))
+            raise ValueError(
+                "Expected method in 0-15 range" " got {!r}".format(method)
+            )
         self.method = method
 
         # tune
@@ -346,7 +443,9 @@ class DefaultArgs:
         # school
         if school not in (0, 1):  # pragma: no cover
             raise ValueError(
-                "School argument can only be either 0 or 1 got {!r}".format(school)
+                "School argument can only be either 0 or 1 got {!r}".format(
+                    school
+                )
             )
         self.school = school
 
@@ -379,6 +478,9 @@ class DefaultArgs:
             "adjustment": self.adjustment,
         }
 
+    def __hash__(self):  # pragma: no cover
+        return hash(tuple(self.as_dict.values()))
+
 
 class Meta:
     """Represents the meta that is in returned :class:`Data`
@@ -396,7 +498,7 @@ class Meta:
         latitude: :class:`float`
             Latitude coordinate.
 
-        timezone:  :class:`UTC`
+        timezone:  :class:`pytz.UTC`
             Used timezone to calculate.
 
         method: :class:`Method`
@@ -434,13 +536,6 @@ class Meta:
         self.school = school
         self.offset = Tune(*offset)
 
-    def __repr__(self):  # pragma: no cover
-        return (
-            "<Meta longitude={0.longitude!r}, latitude={0.latitude!r}, "
-            "method={0.method!r}, latitudeAdjustmentMethod={0.latitudeAdjustmentMethod!r}, "
-            "midnightMode={0.midnightMode!r}, school={0.school!r}, offset={0.offset!r}>"
-        ).format(self)
-
     @property
     def default_args(self):
         """:class:`DefaultArgs`: returns a default args obj"""
@@ -449,10 +544,35 @@ class Meta:
             self.offset,
             getattr(Schools, self.school.upper()),
             getattr(MidnightModes, self.midnightMode.upper()),
-            getattr(LatitudeAdjustmentMethods, self.latitudeAdjustmentMethod.upper())
+            getattr(
+                LatitudeAdjustmentMethods, self.latitudeAdjustmentMethod.upper()
+            )
             # can't get adjustment ...
         )
 
+    def __repr__(self):  # pragma: no cover
+        return (
+            "<Meta longitude={0.longitude!r}, latitude={0.latitude!r}, "
+            "method={0.method!r}, latitudeAdjustmentMethod="
+            "{0.latitudeAdjustmentMethod!r}, midnightMode={0.midnightMode!r}, "
+            "school={0.school!r}, offset={0.offset!r}>"
+        ).format(self)
+
+    def __hash__(self):  # pragma: no cover
+        return hash(
+            (
+                self.method,
+                self.longitude,
+                self.latitude,
+                self.timezone,
+                self.method,
+                self.latitudeAdjustmentMethod,
+                self.method,
+                self.school,
+                self.offset,
+            )
+        )
+
 
 class DateType:
     """A class for gregorian/hijri date.
@@ -471,16 +591,22 @@ class DateType:
             Date's format
 
         day: :class:`int`
+            Date's day.
 
         weekday: dict[:class:`str`, :class:`str`]
+            A dict with 2 keys, "en" and "ar" for hijri and only 1 key "en" for gregorian.
 
         month: dict[:class:`str`, :class:`int` or :class:`str`]
+            A dict with 3 keys "number", "en", "ar" for hijri and 2 keys "number", "en" for gregorian.
 
         year: :class:`int`
+            Date's year.
 
         designation: dict[:class:`str`, :class:`str`]
+            A dict with 2 keys, "abbreviated" and "expanded".
 
-        holidays: :class:`list` of :class:`str`
+        holidays: Optional[:class:`list` of :class:`str`]
+            A list of holidays might be empty for hijri, always None for gregorian.
     """
 
     def __init__(
@@ -507,10 +633,12 @@ class DateType:
 
     def __repr__(self):  # pragma: no cover
         return (
-            "<DateType name={0.name!r}, date={0.date!r}, holidays={0.holidays}>".format(
-                self
-            )
-        )
+            "<DateType name={0.name!r}, date={0.date!r}, "
+            "holidays={0.holidays}>"
+        ).format(self)
+
+    def __hash__(self):  # pragma: no cover
+        return hash((self.name, self.date))
 
 
 class Date:
@@ -556,6 +684,9 @@ class Date:
             "gregorian={0.gregorian!r}, hijri={0.hijri!r}>".format(self)
         )
 
+    def __hash__(self):  # pragma: no cover
+        return hash(self.timestamp)
+
 
 class Timings:
     """Represents the timings that is in returned :class:`Data`
@@ -590,6 +721,8 @@ class Timings:
 
         midnight: :class:`Prayer`
             Midnight time.
+
+    *New in v0.1.4: __iter__*
     """
 
     def __init__(
@@ -617,9 +750,26 @@ class Timings:
         self.isha: Prayer = _Prayer("Isha", Isha)
         self.midnight: Prayer = _Prayer("Midnight", Midnight)
 
+    @property
+    def as_dict(self) -> Dict[str, Prayer]:
+        """dict[:class:`str`, :class:`Prayer`]: A dict of all 5 prayers and the other times
+
+        *New in v0.1.4*"""
+        return {
+            "Imsak": self.imsak,
+            "Fajr": self.fajr,
+            "Sunrise": self.sunrise,
+            "Dhuhr": self.dhuhr,
+            "Asr": self.asr,
+            "Sunset": self.sunset,
+            "Maghrib": self.maghrib,
+            "Isha": self.isha,
+            "Midnight": self.midnight,
+        }
+
     @property
     def prayers_only(self) -> Dict[str, Prayer]:
-        """dict[:class:`str`, :class:`Prayer`]: A dict of only 5 prayers."""
+        """dict[:class:`str`, :class:`Prayer`]: A dict of the 5 prayers."""
         return {
             "Fajr": self.fajr,
             "Dhuhr": self.dhuhr,
@@ -630,13 +780,13 @@ class Timings:
 
     async def next_prayer(self):
         """
-        Get the next coming prayer.
+        Get the next upcoming prayer.
         Don't use this for old dates.
 
         Returns
         -------
             :class:`Prayer`
-                The coming prayer.
+                The upcoming prayer.
         """
         meta = self.data.meta
         now = datetime.utcnow()
@@ -650,13 +800,18 @@ class Timings:
                 meta.longitude,
                 meta.latitude,
                 TimingsDateArg(
-                    datetime(val.time.year, val.time.month, val.time.day)  # noqa
+                    datetime(
+                        val.time.year, val.time.month, val.time.day  # noqa
+                    )
                     + timedelta(1)
                 ),
                 meta.default_args,
             )
         ).next_prayer()
 
+    def __iter__(self) -> Iterable[Prayer]:
+        yield from self.as_dict.values()
+
     def __repr__(self):  # pragma: no cover
         return (
             "<Timings imsak={0.imsak}, fajr={0.fajr}, sunrise={0.sunrise}, "
@@ -664,6 +819,9 @@ class Timings:
             "isha={0.isha}, midnight={0.midnight}>"
         ).format(self)
 
+    def __hash__(self):  # pragma: no cover
+        return hash(tuple(self.as_dict.values()))
+
 
 class Data:
     """Main class Representing the data returned from a request to APi
@@ -693,3 +851,6 @@ class Data:
 
     def __repr__(self):  # pragma: no cover
         return "<Data object | {0.gregorian.date}>".format(self.date)
+
+    def __hash__(self):  # pragma: no cover
+        return hash((self.meta, self.date))
