@@ -3,7 +3,7 @@ import aiohttp
 import platform
 
 from beartype import beartype
-from typing import Union, Dict, List, Tuple, Any
+from typing import Union, Dict, List, Optional
 
 from .methods import all_methods
 from .base_types import *
@@ -42,24 +42,24 @@ class AsyncClient:
     def __init__(self, loop: asyncio.AbstractEventLoop = None):
         self._loop = loop or asyncio.get_event_loop()
 
-    async def __get_res(
-        self,
-        endpoint: str,
-        params: Tuple[
-            Union[str, Any], ...
-        ],  # using tuple cause dict isn't hashable to cache
-    ) -> Union[Timings, List[Timings], Dict[str, List[Timings]]]:
+    async def _get_res(self, endpoint: str, params: dict) -> dict:
         async with aiohttp.ClientSession(loop=self._loop) as session:
-            res = await session.get(endpoint, params=dict(params))
+            res = await session.get(endpoint, params=params)
             res = await res.json()
-        data = res["data"]
 
-        if isinstance(data, str):  # something wrong
-            raise Exception(data)
-        elif isinstance(data, list):  # it is a month calendar
+        if res["code"] != 200:  # something wrong
+            raise Exception("{code}, {data}".format(**res))
+        return res
+
+    async def _get_timings(
+        self, endpoint: str, params: dict
+    ) -> Union[Timings, List[Timings], Dict[str, List[Timings]]]:
+
+        data = (await self._get_res(endpoint, params))["data"]
+
+        if isinstance(data, list):  # it is a month calendar
             return [Data(**day, client=self).timings for day in data]
         # it is a dict
-
         if "1" in data:  # it is a year calendar
             return {
                 month: [Data(**day, client=self).timings for day in days]
@@ -72,8 +72,8 @@ class AsyncClient:
         self,
         longitude: Union[int, float],
         latitude: Union[int, float],
-        date: TimingsDateArg = None,
-        defaults: DefaultArgs = None,
+        date: Optional[TimingsDateArg] = None,
+        defaults: Optional[DefaultArgs] = None,
     ):
         """|coro|
 
@@ -87,11 +87,11 @@ class AsyncClient:
             latitude: :class:`int` or :class:`float`
                 Latitude coordinate of the location.
 
-            date: :class:`TimingsDateArg`
+            date: Optional[:class:`TimingsDateArg`]
                 Date for the prayer times.
                 Default: Current date.
 
-            defaults: :class:`DefaultArgs`
+            defaults: Optional[:class:`DefaultArgs`]
                 Default params.
                 Default: ``DefaultArgs()``
 
@@ -106,16 +106,16 @@ class AsyncClient:
         }
         date, defaults = date or TimingsDateArg(), defaults or DefaultArgs()
         params.update(defaults.as_dict)
-        return await self.__get_res(
-            EndPoints.TIMINGS + "/" + date.date, tuple(params.items())
+        return await self._get_timings(
+            EndPoints.TIMINGS + "/" + date.date, params
         )
 
     @beartype
     async def get_timings_by_address(
         self,
         address: str,
-        date: TimingsDateArg = None,
-        defaults: DefaultArgs = None,
+        date: Optional[TimingsDateArg] = None,
+        defaults: Optional[DefaultArgs] = None,
     ):
         """|coro|
         Get prayer times from address.
@@ -126,11 +126,11 @@ class AsyncClient:
                 An address string.
                 Example: "London, United Kingdom"
 
-            date: :class:`TimingsDateArg`
+            date: Optional[:class:`TimingsDateArg`]
                 Date for the prayer times.
                 Default: Current date.
 
-            defaults: :class:`DefaultArgs`
+            defaults: Optional[:class:`DefaultArgs`]
                 Default params.
                 Default: ``DefaultArgs()``
 
@@ -144,9 +144,8 @@ class AsyncClient:
         }
         date, defaults = date or TimingsDateArg(), defaults or DefaultArgs()
         params.update(defaults.as_dict)
-        return await self.__get_res(
-            EndPoints.TIMINGS_BY_ADDRESS + "/" + date.date,
-            tuple(params.items()),
+        return await self._get_timings(
+            EndPoints.TIMINGS_BY_ADDRESS + "/" + date.date, params
         )
 
     @beartype
@@ -154,9 +153,9 @@ class AsyncClient:
         self,
         city: str,
         country: str,
-        state: str = None,
-        date: TimingsDateArg = None,
-        defaults: DefaultArgs = None,
+        state: Optional[str] = None,
+        date: Optional[TimingsDateArg] = None,
+        defaults: Optional[DefaultArgs] = None,
     ):
         """|coro|
         Get prayer times from city, country and state.
@@ -175,11 +174,11 @@ class AsyncClient:
                 State or province. The state name or abbreviation..
                 Example: "Bexley"
 
-            date: :class:`TimingsDateArg`
+            date: Optional[:class:`TimingsDateArg`]
                 Date for the prayer times.
                 Default: Current date.
 
-            defaults: :class:`DefaultArgs`
+            defaults: Optional[:class:`DefaultArgs`]
                 Default params.
                 Default: ``DefaultArgs()``
 
@@ -197,8 +196,8 @@ class AsyncClient:
             del params["state"]
         date, defaults = date or TimingsDateArg(), defaults or DefaultArgs()
         params.update(defaults.as_dict)
-        return await self.__get_res(
-            EndPoints.TIMINGS_BY_CITY + "/" + date.date, tuple(params.items())
+        return await self._get_timings(
+            EndPoints.TIMINGS_BY_CITY + "/" + date.date, params
         )
 
     @beartype
@@ -207,11 +206,12 @@ class AsyncClient:
         longitude: Union[int, float],
         latitude: Union[int, float],
         date: CalendarDateArg,
-        defaults: DefaultArgs = None,
+        defaults: Optional[DefaultArgs] = None,
     ):
         """|coro|
 
-        Get all prayer times for a specific calendar month/year from coordinates (longitude, latitudes).
+        Get all prayer times for a specific calendar month/year from \
+        coordinates (longitude, latitudes).
 
         Parameters
         -----------
@@ -224,14 +224,16 @@ class AsyncClient:
             date: :class:`CalendarDateArg`
                 Date for the prayer times.
 
-            defaults: :class:`DefaultArgs`
+            defaults: Optional[:class:`DefaultArgs`]
                 Default params.
                 Default: ``DefaultArgs()``
 
         Returns
         -------
-            :class:`list` of :class:`Timings` or dict[:class:`str`, :class:`list` of :class:`Timings`]
-                A month calendar if month parameter was given in date argument otherwise a year calendar.
+            :class:`list` of :class:`Timings` or dict[:class:`str`, \
+            :class:`list` of :class:`Timings`]
+                A month calendar if month parameter was given in date argument \
+                otherwise a year calendar.
         """
         params = {
             "longitude": str(longitude),
@@ -240,14 +242,16 @@ class AsyncClient:
         defaults = defaults or DefaultArgs()
         params.update(defaults.as_dict)
         params.update(date.as_dict)
-        return await self.__get_res(
-            getattr(EndPoints, "HIJRI_" * date.hijri + "CALENDAR"),
-            tuple(params.items()),
+        return await self._get_timings(
+            getattr(EndPoints, "HIJRI_" * date.hijri + "CALENDAR"), params
         )
 
     @beartype
     async def get_calendar_by_address(
-        self, address: str, date: CalendarDateArg, defaults: DefaultArgs = None
+        self,
+        address: str,
+        date: CalendarDateArg,
+        defaults: Optional[DefaultArgs] = None,
     ):
         """|coro|
 
@@ -262,22 +266,24 @@ class AsyncClient:
             date: :class:`CalendarDateArg`
                 Date for the prayer times.
 
-            defaults: :class:`DefaultArgs`
+            defaults: Optional[:class:`DefaultArgs`]
                 Default params.
                 Default: ``DefaultArgs()``
 
         Returns
         -------
-            :class:`list` of :class:`Timings` or dict[:class:`str`, :class:`list` of :class:`Timings`]
-                A month calendar if month parameter was given in date argument otherwise a year calendar.
+            :class:`list` of :class:`Timings` or dict[:class:`str`, \
+            :class:`list` of :class:`Timings`]
+                A month calendar if month parameter was given in date argument \
+                otherwise a year calendar.
         """
         params = {"address": address}
         defaults = defaults or DefaultArgs()
         params.update(defaults.as_dict)
         params.update(date.as_dict)
-        return await self.__get_res(
+        return await self._get_timings(
             getattr(EndPoints, "HIJRI_" * date.hijri + "CALENDAR_BY_ADDRESS"),
-            tuple(params.items()),
+            params,
         )
 
     @beartype
@@ -286,8 +292,8 @@ class AsyncClient:
         city: str,
         country: str,
         date: CalendarDateArg,
-        state: str = None,
-        defaults: DefaultArgs = None,
+        state: Optional[str] = None,
+        defaults: Optional[DefaultArgs] = None,
     ):
         """|coro|
 
@@ -303,21 +309,23 @@ class AsyncClient:
                 The country name or 2 character alpha ISO 3166 code.
                 Example: "GB" or "United Kingdom"
 
+            date: :class:`CalendarDateArg`
+                Date for the prayer times.
+
             state: Optional[:class:`str`]
                 State or province. The state name or abbreviation..
                 Example: "Bexley"
 
-            date: :class:`CalendarDateArg`
-                Date for the prayer times.
-
-            defaults: :class:`DefaultArgs`
+            defaults: Optional[:class:`DefaultArgs`]
                 Default params.
                 Default: ``DefaultArgs()``
 
         Returns
         -------
-            :class:`list` of :class:`Timings` or dict[:class:`str`, :class:`list` of :class:`Timings`]
-                A month calendar if month parameter was given in date argument otherwise a year calendar.
+            :class:`list` of :class:`Timings` or dict[:class:`str`, \
+            :class:`list` of :class:`Timings`]
+                A month calendar if month parameter was given in date argument \
+                otherwise a year calendar.
         """
         params = {
             "city": city,
@@ -329,9 +337,9 @@ class AsyncClient:
         defaults = defaults or DefaultArgs()
         params.update(defaults.as_dict)
         params.update(date.as_dict)
-        return await self.__get_res(
+        return await self._get_timings(
             getattr(EndPoints, "HIJRI_" * date.hijri + "CALENDAR_BY_CITY"),
-            tuple(params.items()),
+            params,
         )
 
     @staticmethod
@@ -341,7 +349,69 @@ class AsyncClient:
 
         Returns
         -------
-            :class:dict[:class:`int`, :class:`Method`]
+            dict[:class:`int`, :class:`Method`]
                 A dict of available calculation method from 0 to 15.
         """
         return all_methods
+
+    @beartype
+    async def get_qibla(
+        self, longitude: Union[int, float], latitude: Union[int, float]
+    ):
+        """|coro|
+
+        Get the Qibla direction from a pair of coordinates.
+
+        Returns
+        -------
+            :class:`Qibla`
+                The qibla.
+
+        *New in v0.1.3*
+        """
+        return Qibla(
+            **(
+                await self._get_res(
+                    EndPoints.QIBLA + "/{}/{}".format(latitude, longitude), {}
+                )
+            )["data"]
+        )
+
+    @beartype
+    async def get_asma(self, *n: int):
+        """|coro|
+
+        Returns a list of asma from giving numbers.
+
+        Returns
+        -------
+            :class:`list` of :class:`Ism`
+                A list of asma.
+
+        *New in v0.1.3*
+        """
+
+        assert n, "No arguments was passed."
+
+        return [
+            Ism(**d)
+            for d in (
+                await self._get_res(
+                    EndPoints.ASMA_AL_HUSNA + "/" + ",".join(map(str, n)), {}
+                )
+            )["data"]
+        ]
+
+    async def get_all_asma(self):
+        """|coro|
+
+        Returns all 1-99 asma (allah names).
+
+        Returns
+        -------
+            :class:`list` of :class:`Ism`
+                A list of all asma.
+
+        *New in v0.1.3*
+        """
+        return await self.get_asma(*range(1, 100))
